#!/usr/bin/python

# Copyright (c) 2007 David Futcher ('bobbo') <bobbocanfly@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, write to:
#     The Free Software Foundation, Inc.,
#     51 Franklin Street, Fifth Floor
#     Boston, MA  02110-1301, USA.
#
#  In addition, as a special exception, the copyright holders give
#  permission to link the code of portions of this program with the OpenSSL
#  library.
#  You must obey the GNU General Public License in all respects for all of
#  the code used other than OpenSSL. If you modify file(s) with this
#  exception, you may extend this exception to your version of the file(s),
#  but you are not obligated to do so. If you do not wish to do so, delete
#  this exception statement from your version. If you delete this exception
#  statement from all source files in the program, then also delete it here.

NAME = "ckkern"
FULLNAME = "Ckkern"
VERSION = "0.1.5"
AUTHOR = "David Futcher"
EMAIL = "bobbocanfly@gmail.com"
DESCRIPTION = "Check the latest kernel number against your currently installed kernel"
URL = "https://launchpad.net/ckkern"
LICENSE = "GPLv2"

import feedparser
import sys
import os
import getopt

#Options file
etcLink = "/etc/ckkern"

bold = False
verbose = False
force = False
feedparser.USER_AGENT = "ckkern " + VERSION + URL

#Prints usage information
def usage():
    print """Ckkern - Check the latest kernel number against your currently installed kernel

Usage:
 ckkern :: Runs in standard mode
 ckkern -h :: Prints this help message
 ckkern -v :: Prints extra information while running (Verbose Mode)
 ckkern -s :: Prints source for kernel information
 ckkern -f :: Forces parsing even if feed is invalid (Feed may not work)

Info:"""
    print " Author: " + AUTHOR + " <" + EMAIL + ">"
    print " Version: " + VERSION
    print " " + URL

#Prints source of kernel information
def source():
    print "New Kernel Source: " + rssLink
    print "Installed Kernel Source: " + krnLink

#Parse settings file
if os.path.exists(etcLink) != True:
    print "-> WARN: Could not find settings file at " + etcLink + ". Using hardcoded defaults."
    rssLink = "http://kernel.org/kdist/rss.xml"
    krnLink = "/proc/version_signature"
else:
    s = open(etcLink)
    for line in s:
        line = str(line).split(';')
        if line[0].lower() == "rsslink":
            rssLink = line[1].strip('\n')
        elif line[0].lower() == "krnlink":
            krnLink = line[1].strip('\n')
        elif line[0].lower() == "forceverbose":
            if line[1].strip('\n') == "1":
                verbose = True
        else:
            print "Ivalid settings file at " + etcLink
            sys.exit(3)

#Parse args
try:
    opts, args = getopt.getopt(sys.argv[1:], "hvsf", ["Help", "Verbose", "Source", "Force Feed Parsing Even If Invalid"])
except getopt.GetoptError:
    usage()
    sys.exit(2)
for o, a in opts:
    if o == "-v":
        verbose = True
    elif o in "-h":
        usage()
        sys.exit()
    elif o in "-s":
        source()
        sys.exit()
    elif o in "-f":
        force = True

if verbose:
    print "-> Downloading RSS feed for parsing"

feed = feedparser.parse(rssLink) #Grab the new feed and parse it

#Check forced parsing
if force == True:
    feed.bozo = 0

#Downloaded, error check it
if feed.bozo == 1 and force == False:
    exc = feed.bozo_exception
    print "-> ERROR: Invalid feed"
    print exc.getMessage() + " at line number " + str(exc.getLineNumber())
    print "-> Run with option '-f' to force parsing (Feedparser may be able to work around error)"
    sys.exit(5)
    

#TODO: Clean this up?
for item in feed["items"]:
    sys.stdout.write("Latest Kernel Version: " + item[ "title" ] + "\n")
    break

if verbose:
    print "-> Fetching current kernel version number"

f = open(krnLink) #Open current kernel file
sys.stdout.write("Installed Kernel Version: " + f.read())
